(* Program to test the svgalib keyboard functions. *)


let newcolor () =
  let bpp = Gl.bytesperpixel (Gl.currentcontext()) in
    if bpp = 1 then
      (Random.int 15) + 1
    else
      Gl.rgbcolor (Random.int 255) (Random.int 255) (Random.int 255)
;;

let timeout signal =
  Keyboard.close();
  Vga.setmode Vga.mode_TEXT;
  Printf.printf "Automatic termination after 60 seconds.";
  exit 1
;;

let rec getchar () =
  let d = Vga.getkey () in
  if d <> 0 then d else getchar()
;;

let main () =
  Vga.init();

  let vgamode = 
    try
      let m = Vga.getdefaultmode() in
    	if (Vga.getmodeinfo m).Vga.bytesperpixel <> 1 then
	  failwith "caught"
	else
	  m
    with Failure _ -> Vga.mode_G320x200x256
  in

    if not (Vga.hasmode vgamode) then begin
      Printf.printf("Mode not available.\n");
      exit 1
    end ;
    List.iter print_string
          ["\nWARNING: This program will set the keyboard to RAW mode.\n";
	   "The keyboard routines in svgalib have not been tested\n";
	   "very much. There may be no recovery if something goes\n";
	   "wrong.\n\n";
	   "Press ctrl-c now to bail out, enter to continue.\n";
	   "In the test itself, use 'q' or Escape to quit.\n";
	   "It will also terminate after 60 seconds.\n";
    "Use any cursor keys to move, keypad 0 or enter to change color.\n"];
    flush stdout;

    getchar();

    let infos = Vga.getmodeinfo vgamode in
    let width, height, colors = infos.Vga.width, infos.Vga.height, 
                                infos.Vga.colors in

    Vga.setmode vgamode;
    Gl.setcontextvga vgamode;
    Gl.enableclipping();

    (* Sys.signal Sys.sigint (Sys.Signal_handle timeout); *) 

    (* This installs the default handler, which is good enough for most *)
    (* purposes. *)
    Keyboard.init();

    (* Translate to 4 keypad cursor keys, and unify enter key. *)
    Keyboard.translatekeys
      [ Keyboard.TRANSLATE_CURSORKEYS ;
        Keyboard.TRANSLATE_KEYPADENTER ;
        Keyboard.TRANSLATE_DIAGONAL ];
    (* (TRANSLATE_DIAGONAL seems to give problems.) Michael: No doesn't...
       but might not do what you expect.. *)

    (* alarm(60); *)		(* Terminate after 60 seconds for safety. *)

      let x = ref (width / 2)
      and y = ref (height / 2)
      and color = ref (newcolor())
      and leftpressed = ref false in

    try
      while true do
	
	(* Draw moving box. *)
	Gl.fillbox !x !y 5 5 !color;

	(* Draw key status bar at top of screen. *)
	Gl.putbox 0 0 128 1 (Keyboard.getstate());

	(* Wait about 1/100th of a second. *)
	(* Note that use of this function makes things less *)
	(* smooth because of timer latency. *)
	for i = 1 to 10000 do () done ;

	Keyboard.update();

	(* Move. *)
	if (Keyboard.keypressed Keyboard.scancode_CURSORLEFT)  then decr x;
	if (Keyboard.keypressed Keyboard.scancode_CURSORRIGHT) then incr x;
	if (Keyboard.keypressed Keyboard.scancode_CURSORUP)    then decr y;
	if (Keyboard.keypressed Keyboard.scancode_CURSORDOWN)  then incr y;

	(* Boundary checks. *)
	if (!x < 0)       then x := 0;
	if (!x >= width)  then x := width - 1;
	if (!y < 1)       then y := 1;
	if (!y >= height) then y := height - 1;

	(* Check for color change. *)
	if (Keyboard.keypressed Keyboard.scancode_KEYPAD0) or
	   (Keyboard.keypressed Keyboard.scancode_ENTER) then
	    if not !leftpressed then begin
	      color := newcolor();
	      leftpressed := true
	    end else ()
	else
	  leftpressed := false;

	if (Keyboard.keypressed Keyboard.scancode_Q) or
	   (Keyboard.keypressed Keyboard.scancode_ESCAPE) then
	     failwith "exit"
	done
	with Failure "exit" -> () ;

    Keyboard.close();		(* Don't forget this! *)

    Vga.setmode Vga.mode_TEXT;
    exit(0);
;;

Printexc.print main ()

