
open Vga;;
open Gl;;

(* The logo was drawn by John Remyn. *)
(* Feel free to provide a more beautiful/official/thought provoking/cool *)
(* logo to replace it. *)
let logowidth = 201
let logoheight = 85

let vgamode = ref mode_TEXT;;
let virtualmode = ref false;;

let backscreen = ref null_graphics_context;;
let physicalscreen = ref null_graphics_context;;

let rec getchar () =
  let d = getkey () in
  if d <> 0 then d else getchar ()
;; 


let loadbitmap filename =
  let buf = String.create (logowidth * logoheight) in
  let i = ref 0 in
  let f = open_in filename in
    begin 
      try
      	while true do
	  let c = input_char f in
	    String.set buf !i c ;
	    incr i
      	done 
      with End_of_file -> () ;
    end ; 
    close_in f ;
    buf
;;

let test () =

  let infos = Vga.getmodeinfo !vgamode in
  let width, height, colors = infos.width, infos.height, infos.colors in
  
    if !virtualmode then
      Gl.setcontext !backscreen;

    Gl.clearscreen 0;
    for i = 1 to 5 do
      Gl.clearscreen 0;
      for j = 1 to  20000 do
      	Gl.setpixel (Random.int width) (Random.int  height) (Random.int colors)
      done
    done ;
    
    if !virtualmode then
      Gl.copyscreen !physicalscreen;

    Gl.clearscreen 0;
    for i = 1 to 500 do
      let x,y = Random.int  (width - 1), Random.int  (height - 1) in
	Gl.fillbox x  y (Random.int  (width - x)) 
	           (Random.int (height - y)) (Random.int colors)
    done ;

    if !virtualmode then
      Gl.copyscreen !physicalscreen;

    Gl.clearscreen 0;
    for i = 1 to 4000 do
      Gl.line (Random.int  width)  (Random.int  height)
	(Random.int  width) (Random.int  height)
	(Random.int  colors)
    done ;

    if !virtualmode then
      Gl.copyscreen !physicalscreen ;

    (* Create bitmap. *)
    let bitmap = String.create (64 * 64 * infos.bytesperpixel) in
    (* Create temporary graphics context to create bitmap in. *)
    let savedcontext = Gl.allocatecontext() in
    Gl.getcontext savedcontext;
    Gl.setcontextvirtual 64 64 infos.bytesperpixel 8 bitmap;
    (* The rgb functions can be used to create nice bitmaps easily for *)
    (* hicolor/truecolor modes. The 256 color 'emulated' truecolor *)
    (* looks less impressive. *)
    for i = 0 to 31 do
      for j = 0 to 31 do
	let c = 255 - (i + j) * 4 in
	  Gl.setpixelrgb (31 - i) (31 - j) (c, 0, 0);
	  Gl.setpixelrgb (32 + i) (31 - j) (c, c, 0);
	  Gl.setpixelrgb (31 - i) (32 + j) (c, 0, c);
	  Gl.setpixelrgb (32 + i) (32 + j) (c, c, c)
      done
    done ;
    (* Restore previous context. *)
    Gl.setcontext savedcontext;

    Gl.clearscreen(0);
    for i = 1 to 4000 do
      let x,y = Random.int  (width - 64 + 1), Random.int  (height - 64 + 1) in
	Gl.putbox x y 64 64 bitmap;
    done ;

    if !virtualmode then
      Gl.copyscreen !physicalscreen
;;


(*
void setcustompalette(void)
{
    /* colors 0-31 are an RGB mix (bits 0 and 1 red, 2 green, 3 and 4 blue) */
    /* 32-63    black to red */
    /* 64-95    black to green */
    /* 96-127   black to yellow */
    /* 128-159  black to blue */
    /* 160-191  black to magenta */
    /* 192-223  black to cyan */
    /* 224-255  black to white */
    Palette pal;
    int i;
    for (i = 0; i < 256; i++) {
	int r, g, b;
	r = g = b = 0;
	if ((i & 32) > 0)
	    r = (i & 31) << 1;
	if ((i & 64) > 0)
	    g = (i & 31) << 1;
	if ((i & 128) > 0)
	    b = (i & 31) << 1;
	if (i < 32) {
	    r = (i & 3) << 4;	/* 2 bits */
	    g = (i & 4) << 3;	/* 1 bit */
	    b = (i & 24) << 1;	/* 2 bits */
	}
	pal.color[i].red = r;
	pal.color[i].green = g;
	pal.color[i].blue = b;
    }
    gl_setpalette(&pal);
}
*)

let logotest () =
  let infos = Vga.getmodeinfo !vgamode in
  let width, height, colors = infos.width, infos.height, infos.colors in
  
    (* setcustompalette(); *)

  let logobitmap = loadbitmap "linuxlogo.bmp" in
    (* Allocate buffer for scaled bitmap. *)
  let scaled = String.create (width * height) in
    Gl.clearscreen 0;
    (* Stretch vertically. *)
    for h = 0 to logoheight do
      Gl.scalebox logowidth logoheight logobitmap logowidth h scaled;
      Gl.putbox 0 0 logowidth h scaled;
      if !virtualmode then
        Gl.copyscreen !physicalscreen;
    done ;
    Gl.clearscreen 0;
    (* Scale to screen resolution. *)
    Gl.scalebox logowidth logoheight logobitmap width height scaled;
    Gl.putbox 0 0 width height scaled;
    Gl.copyscreen !physicalscreen;
;;

let main () =
  init ();
  begin
    try
      vgamode := getdefaultmode()
    with
      	Failure _ -> vgamode := mode_G320x200x256   (* Default mode. *)
  end ;
  if not (hasmode !vgamode) then begin
    Printf.printf "Mode not available.\n";
    exit (-1)
  end ;
  let infos = Vga.getmodeinfo !vgamode in
  let width, height, colors = infos.width, infos.height, infos.colors in
  
  if (colors = 16) or (List.mem IS_MODEX infos.flags) then
    virtualmode := true;
    (* These modes are supported indirectly by vgagl. *)

  if !virtualmode then begin
    (* Create virtual screen. *)
    setcontextvgavirtual !vgamode;
    backscreen := allocatecontext();
    getcontext !backscreen
  end ;
  setmode !vgamode;
  setcontextvga !vgamode;	(* Physical screen context. *)
  physicalscreen := allocatecontext();
  getcontext !physicalscreen;
  if (colors = 256) then
    setrgbpalette();
    
  test();
  
  (* Now do the same with clipping enabled. *)
    Gl.clearscreen 0;
    Gl.setclippingwindow (width / 4) (height / 4) (width - width / 4 - 1)
                         (height - height / 4 - 1);

    test();

    Gl.disableclipping();

    if (colors = 256) then
	(* Show the logo if using 256 color mode. *)
	logotest();

    (* getchar(); *)

    if !virtualmode then
      freecontext !backscreen;
    setmode mode_TEXT;
    exit(0)
;;


try 
  Printexc.print main () 
with _ -> 
  setmode mode_TEXT 
;;

print_newline ();;
flush stdout;;

exit 0;;
